/**********************************************************************
 * Copyright (c) 2014 by SAS Institute Inc., Cary, NC, USA.
 *
 * NAME:          rmbinit
 *
 * PURPOSE:       Initializes macro paths for SAS Risk Management for Banking
 *
 * USAGE:         %rmbinit(VERSION=)
 *
 * PARAMETERS:    VERSION     The product version number
 *
 * NOTES:         All of the metadata server options should have been set prior to calling rmbinit.
 *                This macro is now *always* dependent upon metadata.
 *
 *                May be safely called multiple times -- guarded by the RD_DVD variable.
 *
 *
 *
 *
 **********************************************************************/
%macro rmbinit(VERSION=3.2);

   %global DVD;
   %global PRODUCT;
   %global PRODUCT_FOLDER_NAME;
   %global PRODUCT_VERSION_ID;
   %global RD_PRODNAME;
   %global RD_SWC;
   %global RD_DVD;
   %global RRR_PRODNAME;
   %global RRR_VERSION_NUM;
   %global RRR_CLIENT_LIBREF;
   %global HPRISK_CFG_SWC;
   %global HPRISK_GRID_MODE;

   %*------------------------------------------------------------------;
   %* Identify the product (note: values are case sensitive)           ;
   %*------------------------------------------------------------------;

   %let DVD=rmbcomnsvr;
   %let PRODUCT=rmb;
   %let PRODUCT_FOLDER_NAME=Risk Management for Banking;
   %let PRODUCT_VERSION_ID=&VERSION;
   %let RD_PRODNAME=SAS Risk Management for Banking Server;
   %let RD_SWC=Risk Management for Banking Server &VERSION;
   %let RRR_PRODNAME=Risk Reporting Repository;
   %let RRR_VERSION_NUM=2.21;
   %let RRR_CLIENT_LIBREF=rrr_cli;
   %let HPRISK_CFG_SWC=High-Performance Risk Svr Config 3.2;
   %*------------------------------------------------------------------;
   %* Check whether RD_DVD has already been set so that this logic is  ;
   %* not done twice.  To force the initialization to run again, set   ;
   %* RD_DVD back to empty.                                            ;
   %*------------------------------------------------------------------;

   %if (%bquote(&RD_DVD) eq &DVD) %then %do;
      %return;
   %end;

   %let RD_DVD = &DVD;

   /*set path for deployment root directory*/
   %global DEPLOYMENT_ROOT;
   %global RSK_DEPLOYMENT_ROOT;

   %global RRRDDLPATH;
   %global RRRPROGPATH;

   %global RMX_MACRO_PATHS;
   %global RMX_CUSTOM_PATH;
   %global ORIGINAL_SASAUTOS;

   %let DEPLOYMENT_ROOT=!SASROOT;

   %rsk_get_swc_property(propertyName=risk.source.path, outvar=RSK_DEPLOYMENT_ROOT);
   %rsk_get_swc_property(propertyName=risk.custom.path, outvar=RMX_CUSTOM_PATH);

   /* add our UCMACRO paths to SASAUTOS */
   %let rmx_macro_paths=;

   /* The ucmacros are laid out the same on windows and unix for Ant deployments even though
      in real deployments their paths are different */
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./alm/ucmacros";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./crisk/ucmacros";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./firmwide/ucmacros";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./market/ucmacros";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./common/ucmacros";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./alm/batch";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./crisk/batch";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./firmwide/batch";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./market/batch";
   %let rmx_macro_paths = &rmx_macro_paths "&RSK_DEPLOYMENT_ROOT./common/batch";

   /* prepend the macro paths to SASAUTOS, removing any next parentheses and commas */
   %let ORIGINAL_SASAUTOS= %sysfunc(compress(%sysfunc(getoption(sasautos)),%str(%(%))));

   %local rrr_macro_path ;

   %if ((&SYSSCP eq WIN) or (&SYSSCP eq DNTHOST)) %then %do;
      %let rrrprogpath = !SASROOT/rskrptrpsbk/sasmisc/programs;
      %let rrrddlpath = !SASROOT/rskrptrpsbk/sasmisc/ddls/sas;
      %let rrr_macro_path = !SASROOT/rskrptrpsbk/ucmacros;
   %end;
   %else %do;
      %let rrrprogpath = !SASROOT/misc/rskrptrpsbk/programs;
      %let rrrddlpath= !SASROOT/misc/rskrptrpsbk/ddls/sas;
      %let rrr_macro_path = !SASROOT/ucmacros/rskrptrpsbk;
   %end;

   %local new_autos;
   %let new_autos="&RMX_CUSTOM_PATH"  &RMX_MACRO_PATHS "&RRR_MACRO_PATH" &ORIGINAL_SASAUTOS;
   options sasautos=(&new_autos);
   %put rmbinit set SASAUTOS to: &new_autos;

   /*------------------------------------------------------------------------
    * We/users can now use our product's SAS code
    *------------------------------------------------------------------------*/

   /* Output a message that tells the user what encoding we're in */
   %rsk_print_msg(rsk_system_encoding_note,%sysfunc(getoption(encoding)));

   /*--------------------------------------------------------------------------------------
    * Give access to the lua code we ship
    *--------------------------------------------------------------------------------------*/
   %local misc_dir alm_dir;
   %let misc_dir=%rsk_get_source_dir(component=common);
   %let alm_dir=%rsk_get_source_dir(component=alm);
   filename RISKEXEC (
                      "&RMX_CUSTOM_PATH."
                      "&alm_dir/lua"
                      "&misc_dir/lua"
                      "&misc_dir/lua/utils"
                      );
   /*------------------------------------------------------------------------
    * We can now use our product's Lua code
    *------------------------------------------------------------------------*/

    options noquotelenmax;

   %*------------------------------------------------------------------;
   %* Call the %rmb_post_init macro to perform any specific startup    ;
   %* tasks.  This would be the place to put other paths in the        ;
   %* SASAUTOS autocall path.                                          ;
   %*------------------------------------------------------------------;
   %rmb_post_init(VERSION=&VERSION);

%mend;

%macro rsk_get_swc_property(propertyName=, outvar=_swcprop);
   %local meta_server_up;
   %let   meta_server_up=0;  /* assume the server is down unless we know otherwise */
   data _null_;
      n = 1;
      length uri asn_uri propertyValue$ 500;
      length propertyName $32;
      ver=METADATA_VERSION();
      if ver ne -1 then do;
          call symput("meta_server_up", 1);

          nobj = metadata_getnobj("omsobj:SoftwareComponent?@Name='" || "&rd_swc" || "'", n, uri);
          numasn = metadata_getnasn(uri, "PropertySets", 1, asn_uri);
          numasn = metadata_getnasn(asn_uri, "SetProperties", 1, uri);

          do while(1);
             rc = metadata_getattr(uri, "name", propertyName);
             if (propertyName = "&propertyName") then do;
                rc = metadata_getattr(uri, "DefaultValue", propertyValue);
                call symput("&outvar", trim(propertyValue));
                leave;
             end;
             else do;
                n = n + 1;
                if (n > numasn) then leave;
                rc = metadata_getnasn(asn_uri, "SetProperties", n, uri);
             end;
          end;
     end;
    run;
   %if &meta_server_up ne 1 %then %do;
       %put Metadata server is not available:;
       %let server =%sysfunc(getoption(metaserver));
       %let port =%sysfunc(getoption(metaport));
       %let user = %sysfunc(getoption(metauser));
       %put server=&server port=&port user=&user;
       %put rsk_metadata_server_offline;
       %abort cancel;
   %end;
%mend;
